@@ -3,6 +3,48 @@ import pyqtgraph as pg
 from pyqtgraph import functions as fn
 from pyqtgraph import Point
 import numpy as np
+import pathlib
+
+def make_quadrants(parent):
+    """ make quadrant buttons """
+    parent.quadbtns = QtGui.QButtonGroup(parent)
+    for b in range(9):
+        btn = QuadButton(b, ' '+str(b+1), parent)
+        parent.quadbtns.addButton(btn, b)
+        parent.l0.addWidget(btn, 0 + parent.quadbtns.button(b).ypos, 29 + parent.quadbtns.button(b).xpos, 1, 1)
+        btn.setEnabled(True)
+        b += 1
+    parent.quadbtns.setExclusive(True)
+
+class QuadButton(QtGui.QPushButton):
+    """ custom QPushButton class for quadrant plotting
+        requires buttons to put into a QButtonGroup (parent.quadbtns)
+         allows only 1 button to pressed at a time
+    """
+    def __init__(self, bid, Text, parent=None):
+        super(QuadButton,self).__init__(parent)
+        self.setText(Text)
+        self.setCheckable(True)
+        self.setStyleSheet(parent.styleUnpressed)
+        self.setFont(QtGui.QFont("Arial", 8, QtGui.QFont.Bold))
+        self.resize(self.minimumSizeHint())
+        self.setMaximumWidth(22)
+        self.xpos = bid%3
+        self.ypos = int(np.floor(bid/3))
+        self.clicked.connect(lambda: self.press(parent, bid))
+        self.show()
+
+    def press(self, parent, bid):
+        for b in range(9):
+            if parent.quadbtns.button(b).isEnabled():
+                parent.quadbtns.button(b).setStyleSheet(parent.styleUnpressed)
+        self.setStyleSheet(parent.stylePressed)
+        self.xrange = np.array([self.xpos-.2, self.xpos+1.2]) * parent.Lx/3
+        self.yrange = np.array([self.ypos-.2, self.ypos+1.2]) * parent.Ly/3
+        # change the zoom
+        parent.p0.setXRange(self.xrange[0], self.xrange[1])
+        parent.p0.setYRange(self.yrange[0], self.yrange[1])
+        parent.show()
 
 def horizontal_slider_style():
     return """QSlider::groove:horizontal {
@@ -62,6 +104,22 @@ def horizontal_slider_style():
             border-radius: 4px;
             }"""
 
+class ExampleGUI(QtGui.QDialog):
+    def __init__(self, parent=None):
+        super(ExampleGUI, self).__init__(parent)
+        self.setGeometry(100,100,1300,900)
+        self.setWindowTitle('GUI layout')
+        self.win = QtGui.QWidget(self)
+        layout = QtGui.QGridLayout()
+        self.win.setLayout(layout)
+        guip_path = pathlib.Path.home().joinpath('.cellpose', 'cellpose_gui.png')
+        guip_path = str(guip_path.resolve())
+        pixmap = QtGui.QPixmap(guip_path)
+        label = QtGui.QLabel(self)
+        label.setPixmap(pixmap)
+        pixmap.scaled
+        layout.addWidget(label, 0, 0, 1, 1)
+
 class HelpWindow(QtGui.QDialog):
     def __init__(self, parent=None):
         super(HelpWindow, self).__init__(parent)
@@ -70,6 +128,7 @@ class HelpWindow(QtGui.QDialog):
         self.win = QtGui.QWidget(self)
         layout = QtGui.QGridLayout()
         self.win.setLayout(layout)
+        
         text = ('''
             <p class="has-line-data" data-line-start="5" data-line-end="6">Main GUI mouse controls:</p>
             <ul>
@@ -101,6 +160,10 @@ class HelpWindow(QtGui.QDialog):
             <td>undo previously drawn mask/stroke</td>
             </tr>
             <tr>
+            <td>CTRL+Y</td>
+            <td>undo remove mask</td>
+            </tr>
+            <tr>
             <td>CTRL+0</td>
             <td>clear all masks</td>
             </tr>
@@ -194,7 +257,7 @@ class RGBRadioButtons(QtGui.QButtonGroup):
         super(RGBRadioButtons, self).__init__()
         parent.color = 0
         self.parent = parent
-        self.bstr = ["image", "flowsXY", "cellprob", "flowsZ"]
+        self.bstr = ["image", "gradXY", "cellprob", "gradZ"]
         #self.buttons = QtGui.QButtonGroup()
         self.dropdown = []
         for b in range(len(self.bstr)):
