@@ -1,5 +1,6 @@
 #from suite2p import nonrigid
 import numpy as np
+import warnings
 import cv2
 
 def _taper_mask(bsize=224, sig=7.5):
@@ -9,14 +10,17 @@ def _taper_mask(bsize=224, sig=7.5):
     mask = mask * mask[:, np.newaxis]
     return mask
 
-def unaugment_tiles(y):
+def unaugment_tiles(y, unet=False):
     """ reverse test-time augmentations for averaging
 
     Parameters
     ----------
 
     y: float32
-        array that's ntiles_y x ntiles_x x chan x Ly x Lx where chan = (dY, dX, cell prob);
+        array that's ntiles_y x ntiles_x x chan x Ly x Lx where chan = (dY, dX, cell prob)
+
+    unet: bool (optional, False)
+        whether or not unet output or cellpose output
     
     Returns
     -------
@@ -28,14 +32,17 @@ def unaugment_tiles(y):
         for i in range(y.shape[1]):
             if j%2==0 and i%2==1:
                 y[j,i] = y[j,i, :,::-1, :]
-                y[j,i,0] *= -1
+                if not unet:
+                    y[j,i,0] *= -1
             elif j%2==1 and i%2==0:
                 y[j,i] = y[j,i, :,:, ::-1]
-                y[j,i,1] *= -1
+                if not unet:
+                    y[j,i,1] *= -1
             elif j%2==1 and i%2==1:
                 y[j,i] = y[j,i, :,::-1, ::-1]
-                y[j,i,0] *= -1
-                y[j,i,1] *= -1
+                if not unet:
+                    y[j,i,0] *= -1
+                    y[j,i,1] *= -1
     return y
 
 def average_tiles(y, ysub, xsub, Ly, Lx):
@@ -44,7 +51,7 @@ def average_tiles(y, ysub, xsub, Ly, Lx):
     Parameters
     -------------
 
-    y: float, [ntiles x 3 x bsize x bsize]
+    y: float, [ntiles x nclasses x bsize x bsize]
         output of cellpose network for each tile
 
     ysub : list
@@ -64,12 +71,12 @@ def average_tiles(y, ysub, xsub, Ly, Lx):
     Returns
     -------------
 
-    yf: float32, [3 x Ly x Lx]
+    yf: float32, [nclasses x Ly x Lx]
         network output averaged over tiles
 
     """
     Navg = np.zeros((Ly,Lx))
-    yf = np.zeros((3, Ly, Lx), np.float32)
+    yf = np.zeros((y.shape[1], Ly, Lx), np.float32)
     # taper edges of tiles
     mask = _taper_mask(bsize=y.shape[-1])
     for j in range(len(ysub)):
@@ -125,8 +132,8 @@ def make_tiles(imgi, bsize=224, augment=True):
 
     if augment:
         # tiles overlap by half of tile size
-        ny = max(2, int(np.ceil(1.75 * Ly / bsize)))
-        nx = max(2, int(np.ceil(1.75 * Lx / bsize)))
+        ny = max(2, int(np.ceil(2. * Ly / bsize)))
+        nx = max(2, int(np.ceil(2. * Lx / bsize)))
         ystart = np.linspace(0, Ly-bsize, ny).astype(int)
         xstart = np.linspace(0, Lx-bsize, nx).astype(int)
 
@@ -171,13 +178,14 @@ def normalize99(img):
     X = (X - np.percentile(X, 1)) / (np.percentile(X, 99) - np.percentile(X, 1))
     return X
 
-def reshape(data, channels=[0,0], invert=False):
-    """ reshape data using channels and normalize intensities (w/ optional inversion)
+def reshape(data, channels=[0,0], chan_first=False):
+    """ reshape data using channels
 
     Parameters
     ----------
 
     data : numpy array that's (Z x ) Ly x Lx x nchan
+        if data.ndim==8 and data.shape[0]<8, assumed to be nchan x Ly x Lx
 
     channels : list of int of length 2 (optional, default [0,0])
         First element of list is the channel to segment (0=grayscale, 1=red, 2=blue, 3=green).
@@ -190,28 +198,22 @@ def reshape(data, channels=[0,0], invert=False):
 
     Returns
     -------
-    data : numpy array that's nchan x (Z x ) Ly x Lx
+    data : numpy array that's (Z x ) Ly x Lx x nchan (if chan_first==False)
 
     """
     data = data.astype(np.float32)
     if data.ndim < 3:
         data = data[:,:,np.newaxis]
     elif data.shape[0]<8 and data.ndim==3:
-        data = np.transpose(data, (1,2,0))
+        data = np.transpose(data, (1,2,0))    
 
     # use grayscale image
     if data.shape[-1]==1:
-        data = normalize99(data)
-        if invert:
-            data = -1*data + 1
         data = np.concatenate((data, np.zeros_like(data)), axis=-1)
     else:
         if channels[0]==0:
             data = data.mean(axis=-1)
             data = np.expand_dims(data, axis=-1)
-            data = normalize99(data)
-            if invert:
-                data = -1*data + 1
             data = np.concatenate((data, np.zeros_like(data)), axis=-1)
         else:
             chanid = [channels[0]-1]
@@ -219,46 +221,90 @@ def reshape(data, channels=[0,0], invert=False):
                 chanid.append(channels[1]-1)
             data = data[...,chanid]
             for i in range(data.shape[-1]):
-                if np.ptp(data[...,i]) > 0.0:
-                    data[...,i] = normalize99(data[...,i])
-                else:
+                if np.ptp(data[...,i]) == 0.0:
                     if i==0:
-                        print("WARNING: 'chan to seg' has value range of ZERO")
-                    #else:
-                    #    print("WARNING: 'chan2 (opt)' has value range of ZERO, can instead set chan2 to 0")
-            
-    if data.ndim==4:
-        data = np.transpose(data, (3,0,1,2))
-    else:
-        data = np.transpose(data, (2,0,1))
+                        warnings.warn("chan to seg' has value range of ZERO")
+                    else:
+                        warnings.warn("'chan2 (opt)' has value range of ZERO, can instead set chan2 to 0")
+            if data.shape[-1]==1:
+                data = np.concatenate((data, np.zeros_like(data)), axis=-1)
+    if chan_first:
+        if data.ndim==4:
+            data = np.transpose(data, (3,0,1,2))
+        else:
+            data = np.transpose(data, (2,0,1))
     return data
 
-def normalize_img(img):
+def normalize_img(img, axis=-1, invert=False):
     """ normalize each channel of the image so that so that 0.0=1st percentile
     and 1.0=99th percentile of image intensities
 
+    optional inversion
+
     Parameters
     ------------
 
-    img: ND-array
-        image of size [nchan x Ly x Lx]
+    img: ND-array (at least 3 dimensions)
+
+    axis: channel axis to loop over for normalization
 
     Returns
     ---------------
 
     img: ND-array, float32
-        normalized image of size [nchan x Ly x Lx]
+        normalized image of same size
 
     """
+    if img.ndim<3:
+        raise ValueError('Image needs to have at least 3 dimensions')
+
     img = img.astype(np.float32)
+    img = np.moveaxis(img, axis, 0)
     for k in range(img.shape[0]):
         if np.ptp(img[k]) > 0.0:
             img[k] = normalize99(img[k])
+            if invert:
+                img[k] = -1*img[k] + 1   
+    img = np.moveaxis(img, 0, axis)
     return img
 
-def reshape_data(train_data, test_data=None, channels=None):
-    """ inputs converted to correct shapes for training and rescaled so that 0.0=1st percentile
-    and 1.0=99th percentile of image intensities in each channel.
+def reshape_train_test(train_data, train_labels, test_data, test_labels, channels, normalize):
+    """ check sizes and reshape train and test data for training """
+    nimg = len(train_data)
+    # check that arrays are correct size
+    if nimg != len(train_labels):
+        raise ValueError('train data and labels not same length')
+        return
+    if train_labels[0].ndim < 2 or train_data[0].ndim < 2:
+        raise ValueError('training data or labels are not at least two-dimensional')
+        return
+
+    if train_data[0].ndim > 3:
+        raise ValueError('training data is more than three-dimensional (should be 2D or 3D array)')
+        return
+
+    # check if test_data correct length
+    if not (test_data is not None and test_labels is not None and
+            len(test_data) > 0 and len(test_data)==len(test_labels)):
+        test_data = None
+
+    # make data correct shape and normalize it so that 0 and 1 are 1st and 99th percentile of data
+    train_data, test_data, run_test = reshape_and_normalize_data(train_data, test_data=test_data, 
+                                                                 channels=channels, normalize=normalize)
+
+    if train_data is None:
+        raise ValueError('training data do not all have the same number of channels')
+        return
+
+    if not run_test:
+        print('NOTE: test data not provided OR labels incorrect OR not same number of channels as train data')
+        test_data, test_labels = None, None
+
+    return train_data, train_labels, test_data, test_labels, run_test
+
+def reshape_and_normalize_data(train_data, test_data=None, channels=None, normalize=True):
+    """ inputs converted to correct shapes for *training* and rescaled so that 0.0=1st percentile
+    and 1.0=99th percentile of image intensities in each channel
 
     Parameters
     --------------
@@ -275,6 +321,9 @@ def reshape_data(train_data, test_data=None, channels=None):
         For instance, to train on grayscale images, input [0,0]. To train on images with cells
         in green and nuclei in blue, input [2,3].
 
+    normalize: bool (optional, True)
+        normalize data so 0.0=1st percentile and 1.0=99th percentile of image intensities in each channel
+
     Returns
     -------------
 
@@ -292,9 +341,9 @@ def reshape_data(train_data, test_data=None, channels=None):
     # if training data is less than 2D
     nimg = len(train_data)
     if channels is not None:
-        train_data = [reshape(train_data[n], channels=channels) for n in range(nimg)]
+        train_data = [reshape(train_data[n], channels=channels, chan_first=True) for n in range(nimg)]
     if train_data[0].ndim < 3:
-        train_data = [train_data[n][np.newaxis,:,:] for n in range(nimg)]
+        train_data = [train_data[n][:,:,np.newaxis] for n in range(nimg)]
     elif train_data[0].shape[-1] < 8:
         print('NOTE: assuming train_data provided as Ly x Lx x nchannels, transposing axes to put channels first')
         train_data = [np.transpose(train_data[n], (2,0,1)) for n in range(nimg)]
@@ -306,34 +355,31 @@ def reshape_data(train_data, test_data=None, channels=None):
     # check for valid test data
     run_test = False
     if test_data is not None:
-        nimg = len(test_data)
+        nimgt = len(test_data)
         if channels is not None:
-            test_data = [reshape(test_data[n], channels=channels) for n in range(nimg)]
+            test_data = [reshape(test_data[n], channels=channels, chan_first=True) for n in range(nimgt)]
         if test_data[0].ndim==2:
             if nchan==1:
                 run_test = True
-                test_data = [test_data[n][np.newaxis,:,:] for n in range(nimg)]
+                test_data = [test_data[n][np.newaxis,:,:] for n in range(nimgt)]
         elif test_data[0].ndim==3:
             if test_data[0].shape[-1] < 8:
                 print('NOTE: assuming test_data provided as Ly x Lx x nchannels, transposing axes to put channels first')
-                test_data = [np.transpose(test_data[n], (2,0,1)) for n in range(nimg)]
-            nchan_test = [test_data[n].shape[0] for n in range(nimg)]
+                test_data = [np.transpose(test_data[n], (2,0,1)) for n in range(nimgt)]
+            nchan_test = [test_data[n].shape[0] for n in range(nimgt)]
             if nchan_test.count(nchan_test[0]) != len(nchan_test):
                 run_test = False
             elif test_data[0].shape[0]==nchan:
                 run_test = True
-
-    # normalize_data if no channels given
-    if channels is None:
-        nimg = len(train_data)
-        train_data = [normalize_img(train_data[n]) for n in range(nimg)]
+    
+    if normalize:
+        train_data = [normalize_img(train_data[n], axis=0) for n in range(nimg)]
         if run_test:
-            nimg = len(test_data)
-            test_data = [normalize_img(test_data[n]) for n in range(nimg)]
+            test_data = [normalize_img(test_data[n], axis=0) for n in range(nimgt)]
 
     return train_data, test_data, run_test
 
-def resize_image(img0, Ly, Lx):
+def resize_image(img0, Ly=None, Lx=None, rsz=None, interpolation=cv2.INTER_LINEAR):
     """ resize image for computing flows / unresize for computing dynamics
 
     Parameters
@@ -342,6 +388,15 @@ def resize_image(img0, Ly, Lx):
     img0: ND-array
         image of size [y x x x nchan] or [Lz x y x x x nchan]
 
+    Ly: int, optional
+
+    Lx: int, optional
+
+    rsz: float, optional
+        resize coefficient(s) for image; if Ly is None then rsz is used
+
+    interpolation: cv2 interp method (optional, default cv2.INTER_LINEAR)
+
     Returns
     --------------
 
@@ -349,12 +404,22 @@ def resize_image(img0, Ly, Lx):
         image of size [Ly x Lx x nchan] or [Lz x Ly x Lx x nchan]
 
     """
+    if Ly is None and rsz is None:
+        raise ValueError('must give size to resize to or factor to use for resizing')
+
+    if Ly is None:
+        # determine Ly and Lx using rsz
+        if not isinstance(rsz, list) or not isinstance(rsz, np.ndarray):
+            rsz = [rsz, rsz]
+        Ly = int(img0.shape[-3] * rsz[-2])
+        Lx = int(img0.shape[-2] * rsz[-1])
+    
     if img0.ndim==4:
         imgs = np.zeros((img0.shape[0], Ly, Lx, img0.shape[-1]), np.float32)
         for i,img in enumerate(img0):
-            imgs[i] = cv2.resize(img, (Lx, Ly))
+            imgs[i] = cv2.resize(img, (Lx, Ly), interpolation=interpolation)
     else:
-        imgs = cv2.resize(img0, (Lx, Ly))
+        imgs = cv2.resize(img0, (Lx, Ly), interpolation=interpolation)
     return imgs
 
 def pad_image_ND(img0, div=16, extra = 1):
@@ -400,7 +465,8 @@ def pad_image_ND(img0, div=16, extra = 1):
     xsub = np.arange(ypad1, ypad1+Lx)
     return I, ysub, xsub
 
-def random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224), do_flip=True, rescale=None):
+def random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224), 
+                             do_flip=True, rescale=None, unet=False):
     """ augmentation by random rotation and resizing
 
         X and Y are lists or arrays of length nimg, with dims channels x Ly x Lx (channels optional)
@@ -413,7 +479,8 @@ def random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224), do_flip=
         Y: LIST of ND-arrays, float (optional, default None)
             list of image labels of size [nlabels x Ly x Lx] or [Ly x Lx]. The 1st channel
             of Y is always nearest-neighbor interpolated (assumed to be masks or 0-1 representation).
-            If Y.shape[0]==3, then the labels are assumed to be [cell probability, Y flow, X flow].
+            If Y.shape[0]==3 and not unet, then the labels are assumed to be [cell probability, Y flow, X flow]. 
+            If unet, second channel is dist_to_bound.
 
         scale_range: float (optional, default 1.0)
             Range of resizing of images for augmentation. Images are resized by
@@ -422,12 +489,14 @@ def random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224), do_flip=
         xy: tuple, int (optional, default (224,224))
             size of transformed images to return
 
-        do_flip: bool
+        do_flip: bool (optional, default True)
             whether or not to flip images horizontally
 
         rescale: array, float (optional, default None)
             how much to resize images by before performing augmentations
 
+        unet: bool (optional, default False)
+
         Returns
         -------
         imgi: ND-array, float
@@ -488,7 +557,7 @@ def random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224), do_flip=
             img = img[..., ::-1]
             if Y is not None:
                 labels = labels[..., ::-1]
-                if nt > 1:
+                if nt > 1 and not unet:
                     labels[2] = -labels[2]
 
         for k in range(nchan):
@@ -502,7 +571,7 @@ def random_rotate_and_resize(X, Y=None, scale_range=1., xy = (224,224), do_flip=
                 else:
                     lbl[n,k] = cv2.warpAffine(labels[k], M, (xy[1],xy[0]), flags=cv2.INTER_LINEAR)
 
-            if nt>1:
+            if nt > 1 and not unet:
                 v1 = lbl[n,2].copy()
                 v2 = lbl[n,1].copy()
                 lbl[n,1] = (-v1 * np.sin(-theta) + v2*np.cos(-theta))
